namespace Geekeey.Common.Results.Tests;

[TestFixture]
internal sealed class ResultConversionTests
{
	[Test]
	public void ImplicitConversion_FromValue_IsSuccess()
	{
		var result = Success(2);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.IsFailure, Is.False);
			Assert.That(result.Value, Is.EqualTo(2));
		});
	}

	[Test]
	public void ImplicitConversion_FromError_IsFailure()
	{
		var error = new CustomTestError();
		var result = Failure<int>(error);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.IsFailure, Is.True);
			Assert.That(result.Error, Is.InstanceOf<CustomTestError>());
		});
	}

	[Test]
	public void Unwrap_ReturnsValue_ForSuccess()
	{
		var result = Success(2);
		var value = result.Unwrap();

		Assert.That(value, Is.EqualTo(2));
	}

	[Test]
	public void Unwrap_Throws_ForFailure()
	{
		var result = Failure<int>("error");

		Assert.That(() => result.Unwrap(), Throws.InstanceOf<UnwrapException>());
	}

	[Test]
	public void ExplicitConversion_ReturnsValue_ForSuccess()
	{
		var result = Success(2);
		var value = (int)result;

		Assert.That(value, Is.EqualTo(2));
	}

	[Test]
	public void ExplicitConversion_Throws_ForFailure()
	{
		var result = Failure<int>("error");

		Assert.That(() => (int)result, Throws.InstanceOf<UnwrapException>());
	}
}