namespace Geekeey.Common.Results.Tests;

[TestFixture]
internal sealed class ResultTests
{
	[Test]
	public void New_T_HasValue()
	{
		var result = new Result<int>(1);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.IsFailure, Is.False);
			Assert.That(result.Value, Is.Not.EqualTo(default(int)));
			Assert.That(result.Error, Is.Null);
		});
	}

	[Test]
	public void New_Error_HasError()
	{
		var result = new Result<int>(new CustomTestError());

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.IsFailure, Is.True);
			Assert.That(result.Value, Is.EqualTo(default(int)));
			Assert.That(result.Error, Is.InstanceOf<CustomTestError>());
		});
	}

	[Test]
	public void Default_IsDefault()
	{
		var result = default(Result<int>);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.IsFailure, Is.True);
			Assert.That(result.Value, Is.EqualTo(default(int)));
			Assert.That(result.Error, Is.EqualTo(Error.DefaultValueError));
		});
	}

	[Test]
	public void ToString_ReturnsSuccessString()
	{
		Result<int> result = 2;

		Assert.That(result.ToString(), Is.EqualTo("Success { 2 }"));
	}

	[Test]
	public void ToString_ReturnsFailureString()
	{
		Result<int> result = new StringError("error");

		Assert.That(result.ToString(), Is.EqualTo("Failure { error }"));
	}
}