namespace Geekeey.Common.Results;

public abstract class Error
{
	/// <summary>
	/// A statically accessible default "Result has no value." error.
	/// </summary>
	internal static Error DefaultValueError { get; } = new StringError("The result has no value.");

	/// <summary>
	/// The message used to display the error.
	/// </summary>
	public abstract string Message { get; }

	/// <summary>
	/// Gets a string representation of the error. Returns <see cref="Message"/> by default.
	/// </summary>
	public override string ToString() => Message;

	/// <summary>
	/// Implicitly converts a string into a <see cref="StringError"/>.
	/// </summary>
	/// <param name="message">The message of the error.</param>
	public static implicit operator Error(string message)
		=> new StringError(message);

	/// <summary>
	/// Implicitly converts an exception into an <see cref="ExceptionError"/>.
	/// </summary>
	/// <param name="exception">The exception to convert.</param>
	public static implicit operator Error(Exception exception)
		=> new ExceptionError(exception);
}