using System.Diagnostics.CodeAnalysis;
using System.Runtime.CompilerServices;

namespace Geekeey.Common.Results;

[ExcludeFromCodeCoverage]
public static partial class Extensions
{
	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async Task<Result<TNew>> Map<T, TNew>(this ValueTask<Result<T>> result,
		Func<T, TNew> func)
		=> (await result).Map(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async ValueTask<Result<TNew>> Map<T, TNew>(this Task<Result<T>> result,
		Func<T, TNew> func)
		=> (await result).Map(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async ValueTask<Result<TNew>> Map<T, TNew>(this ValueTask<Result<T>> result,
		Func<T, ValueTask<TNew>> func)
		=> await (await result).MapAsync(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async Task<Result<TNew>> Map<T, TNew>(this Task<Result<T>> result,
		Func<T, ValueTask<TNew>> func)
		=> await (await result).MapAsync(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async ValueTask<Result<TNew>> Then<T, TNew>(this ValueTask<Result<T>> result,
		Func<T, Result<TNew>> func)
		=> (await result).Then(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async Task<Result<TNew>> Then<T, TNew>(this Task<Result<T>> result,
		Func<T, Result<TNew>> func)
		=> (await result).Then(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async ValueTask<Result<TNew>> Then<T, TNew>(this ValueTask<Result<T>> result,
		Func<T, ValueTask<Result<TNew>>> func)
		=> await (await result).ThenAsync(func);

	[MethodImpl(MethodImplOptions.AggressiveInlining)]
	public static async Task<Result<TNew>> Then<T, TNew>(this Task<Result<T>> result,
		Func<T, ValueTask<Result<TNew>>> func)
		=> await (await result).ThenAsync(func);
}